import { Service } from "../lib/service"
import * as async from "async"
import * as mongodb from "mongodb"
export namespace MongodbService {
    export type Settings = {
        database?: {
            port?: number
            host?: string
            name?: string
        }
    }
}
export class MongodbService extends Service<MongodbService.Settings> {
    constructor(settings: MongodbService.Settings) {
        super({
            database: {
                port: 27017,
                host: "localhost",
                name: "test"
            }
        }, settings)
    }
    services: {}
    name = "MongodbService" as "MongodbService"
    db: mongodb.Db
    initialize(done) {
        let {
            host, port, name
        } = this.settings.database
        console.log("Connecting to database", `mongodb://${host}:${port}/${name}`)
        mongodb.MongoClient.connect(`mongodb://${host}:${port}`, (err, client) => {
            this.db = client.db(name)
            done(err)
        })
    }
    asBuiltIn() {
        let {
            host, port, name
        } = this.settings.database
        console.log(`MongodbService using mongo://${host}:${port}/${name}`)
    }
    ensureIndexes(indexes: {
        collection: mongodb.Collection,
        key: any,
        option?: {
            nonunique?: boolean,
            sparse?: boolean
        }
    }[], callback: Callback<null>) {
        async.eachSeries(indexes, (data, done) => {
            if (!data.option) data.option = {}
            console.log(`Indexing ${data.collection.namespace}:${Object.keys(data.key).join("&")} => ${data.option.nonunique && "" || "unique"} ${data.option.sparse && "sparse"}`)
            data.collection.createIndex(data.key, {
                unique: !data.option.nonunique,
                sparse: data.option.sparse
            }, done)
        }, callback)
    }
}

